#!/usr/bin/env python

"""
This is a based on an program that generates sorted playlists for
Iriver T10 UMS Copyright (C) 2006 Dan Fritz

Modified, expanded and tested with an iRiver E100 (c) 2010 by
Michael Fischer v. Mollard
 
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"""

import fnmatch, os, string, struct, sys, getopt, re

def uprint(msg):
    print msg.encode("latin1")

def toUni(msg):
    """
    Sometimes os.listdir(unicode) does not return unicode when it should.
    Instead it returns a mixture of byte strings and unicode - frustrating.
    This method tries to ensure that the string is converted to unicode
    """    
    if isinstance(msg,unicode):
        return msg
    else:
        try:
            return msg.decode("850")
        except Exception:
            pass
        try:
            return msg.decode("latin1")
        except Exception:
            throw       
    
def writePlaylists(root,do_consolidate):
    """
    Recursively scans for music files under the directory "music"
    Creates sorted playlists of each folder with music found.
    """
    root=os.path.normpath(root)
    rootlen = os.path.join(root, u"Music").find(u"Music",len(root))
    playlists = walk(rootlen,os.path.join(root, u"Music"), {})
    result = []
    if do_consolidate:
        consolidate(playlists)

    for dir in playlists:
        playlistfn = os.path.join(root, u"Playlists")
        playlistname = os.path.basename(dir)+u".pla"
        playlistfn = os.path.join(playlistfn,os.path.basename(dir)+u".pla")
        verbose_print("Creating playlist \"%s\"" % (playlistname))
        f = open(playlistfn,"w")
        try:
            line = struct.pack(">l",len(playlists[dir]))
            line += "iriver UMS PLA"
            f.write(line)
            f.write("\0" * (512-len(line)))
            playlists[dir].sort()
            for file in playlists[dir]:
                file="\\" + file
                dirpathlen = len(os.path.dirname(file)) + 2
                file = file.replace("/","\\")
                verbose_print(" Added %s" % (file))

                line = struct.pack("xbx",dirpathlen)
                line += file.encode("utf-16")[2:]
                line += "\0" * (512-len(line))
                f.write(line)
        finally:
            f.close()
        verbose_print("Succesfully created playlist \"%s\"" % (playlistname))
    return result


def walk( rootlen, currdir, result):
    """
    walks through the directory and locates audio-files
    """
    try:
        names = os.listdir(currdir)
    except os.error:
        return result

    pat_list = ["*.mp3","*.ogg","*.wma","*.flac"]    
    for name in names:
        name = toUni(name)
        fullname = os.path.join(currdir, name)
	verbose_print(fullname)
        # grab if it matches our pattern and entry type
        for pat in pat_list:
            if fnmatch.fnmatch(name, pat):
                if os.path.isfile(fullname):
                    dirname = currdir[rootlen:]
                    if dirname not in result:
                        result[dirname]=[]
                    result[dirname].append(fullname[rootlen:])
                continue

        # recursively scan other folders, appending results
        if os.path.isdir(fullname) and not os.path.islink(fullname):
            walk(rootlen, fullname, result )

    return result               # End walk

def consolidate (dirlist):
    """
    Tries to find dirname patterns in the dirlist.  An audiobook may
    have the same 'basename', but followed with e.g. 'cd__x' or
    'cd__x_of_y' In this case, only one entry is written.
    """    
    cd_pattern = re.compile("[ _-]+(cd|disk)?[ _-]?[0-9]+[ _-]?(of|von)?[ _-]?[0-9]?",
                            re.IGNORECASE)
    dirs = dirlist.keys() 
    dirs.sort()
    consolidated_dirs={};
    for dir in dirs:
        consolidated_dir = cd_pattern.sub("",dir)
        verbose_print("dir: "+dir+" consolidated: "+ consolidated_dir)
        if consolidated_dirs.has_key(consolidated_dir):
            consolidated_dirs[consolidated_dir].append(dir);
        else:
            consolidated_dirs[consolidated_dir] = [dir];

    
    for consolidated_dir in consolidated_dirs:
        verbose_print(consolidated_dir+":")
        for cons_val in consolidated_dirs[consolidated_dir]:
            if cons_val == consolidated_dir:
                continue
            dirlist[cons_val].sort()
            if not dirlist.has_key(consolidated_dir):
                dirlist[consolidated_dir]=[]
            for file in dirlist[cons_val]:
                verbose_print("\t"+file)
                dirlist[consolidated_dir].append(file)
            del dirlist[cons_val]
    return 

def usage():
    print """
iRiverPlaylist [-h|--help] -i|--input path [-c|--consolidate]

Generates a playlist suitable for iRiver E100
-i|--input 		Device path (default: '.')
-c|--consolidate	try to build a single playlist for ci series
-h|--help		Print only this message
-v|--verbose		Be verbose


"""    
    return

def verbose_print(msg):
    if verbose:
        uprint(msg)
    return

if __name__ == '__main__':
    "First argument to this script is the path to the Iriver mount"
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hcvi:", 
                                   ["help", "consolidate", "input"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print(str(err)) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    output = None
    input = u"."
    ouput = u"./Playlists"
    verbose = False
    do_consolidate = False
    for o, a in opts:
        if o in ["-v", "--verbose"]:
            verbose = True
        elif o in ("-c", "--consolidate"):
            do_consolidate = True
        elif o in ("-h", "--help"):
            usage()
            sys.exit()
        elif o in ("-i", "--input"):
            input = a
        else:
            assert False, "unhandled option"

    writePlaylists(input,do_consolidate)


